from typing import Any, Dict, Tuple, Callable


def _send_message(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "message/send",
        "POST",
        {
            "peer": {
                "chatType": data["chat_type"],
                "peerUin": data["target"],
                "guildId": None,
            },
            "elements": data["elements"],
        },
    )


def _get_self_profile(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return "getSelfProfile", "GET", {}


def _get_friends(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return "bot/friends", "GET", {}


def _get_groups(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return "bot/groups", "GET", {}


def _mute_member(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/muteMember",
        "POST",
        {
            "group": data["group"],
            "memList": [
                {"uin": i, "timeStamp": data["duration"]} for i in data["members"]
            ],
        },
    )


def _unmute_member(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/muteMember",
        "POST",
        {
            "group": data["group"],
            "memList": [{"uin": i, "timeStamp": 0} for i in data["members"]],
        },
    )


def _mute_everyone(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/muteEveryone",
        "POST",
        {"group": data["group"], "enable": True},
    )


def _unmute_everyone(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/muteEveryone",
        "POST",
        {"group": data["group"], "enable": False},
    )


def _kick(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/kick",
        "POST",
        {
            "uidList": data["members"],
            "group": data["group"],
            "refuseForever": data["refuse_forever"],
            "reason": data["reason"],
        },
    )


def _get_announcements(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return "group/getAnnouncements", "POST", {"group": data["group"]}


def _get_members(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "group/getMemberList",
        "POST",
        {"group": data["group"], "size": data["size"]},
    )


def _fetch_media(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "message/fetchRichMedia",
        "POST",
        {
            "msgId": data["msg_id"],
            "chatType": data["chat_type"],
            "peerUid": data["target"],
            "elementId": data["element_id"],
            "thumbSize": data["thumb_size"],
            "downloadType": data["download_type"],
        },
    )


def _upload(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return "upload", "POST", data["file"]


def _recall_message(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "message/recall",
        "POST",
        {
            "msgIds": data["msg_ids"],
            "peer": {
                "chatType": data["chat_type"],
                "peerUin": data["target"],
                "guildId": None,
            },
        },
    )


def _get_history_messages(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "message/getHistory",
        "POST",
        {
            "peer": {
                "chatType": data["chat_type"],
                "peerUin": data["target"],
                "guildId": None,
            },
            "offsetMsgId": data["offset_msg_id"],
            "count": data["count"],
        },
    )


def _send_fake_forward(data: Dict[str, Any]) -> Tuple[str, str, dict]:
    return (
        "message/unsafeSendForward",
        "POST",
        {
            "dstContact": {
                "chatType": data["chat_type"],
                "peerUin": data["target"],
                "guildId": None,
            },
            "srcContact": {
                "chatType": data["source_chat_type"],
                "peerUin": data["source_target"],
                "guildId": None,
            },
            "msgElements": data["elements"],
        },
    )


HANDLERS: Dict[str, Callable[[Dict[str, Any]], Tuple[str, str, dict]]] = {
    "send_message": _send_message,
    "get_self_profile": _get_self_profile,
    "get_friends": _get_friends,
    "get_groups": _get_groups,
    "mute_member": _mute_member,
    "unmute_member": _unmute_member,
    "mute_everyone": _mute_everyone,
    "unmute_everyone": _unmute_everyone,
    "kick": _kick,
    "get_announcements": _get_announcements,
    "get_members": _get_members,
    "fetch_media": _fetch_media,
    "upload": _upload,
    "recall_message": _recall_message,
    "get_history_messages": _get_history_messages,
    "send_fake_forward": _send_fake_forward,
}



import random
from io import BytesIO
from pathlib import Path
from datetime import datetime
from typing_extensions import override
from dataclasses import field, dataclass
from typing import TYPE_CHECKING, List, Type, Union, Iterable, Optional

from nonebot.exception import NetworkError
from nonebot.internal.driver import Request

from nonebot.adapters import Message as BaseMessage
from nonebot.adapters import MessageSegment as BaseMessageSegment

from .utils import log
from .api.model import Element, UploadResponse

if TYPE_CHECKING:
    from .bot import Bot


class MessageSegment(BaseMessageSegment["Message"]):
    @classmethod
    @override
    def get_message_class(cls) -> Type["Message"]:
        # 返回适配器的 Message 类型本身
        return Message

    @override
    def __str__(self) -> str:
        shown_data = {k: v for k, v in self.data.items() if not k.startswith("_")}
        # 返回该消息段的纯文本表现形式，通常在日志中展示
        return self.data["text"] if self.is_text() else f"[{self.type}: {shown_data}]"

    @override
    def is_text(self) -> bool:
        # 判断该消息段是否为纯文本
        return self.type == "text"

    @staticmethod
    def text(text: str) -> "MessageSegment":
        return MessageSegment("text", {"text": text})

    @staticmethod
    def at(user_id: str, user_name: Optional[str] = None) -> "MessageSegment":
        return MessageSegment("at", {"user_id": user_id, "user_name": user_name})

    @staticmethod
    def at_all() -> "MessageSegment":
        return MessageSegment("at_all")

    @staticmethod
    def image(file: Union[Path, BytesIO, bytes]) -> "MessageSegment":
        if isinstance(file, Path):
            file = file.read_bytes()
        elif isinstance(file, BytesIO):
            file = file.getvalue()
        return MediaMessageSegment("image", {"file": file})

    @staticmethod
    def file(file: Union[Path, BytesIO, bytes]) -> "MessageSegment":
        if isinstance(file, Path):
            file = file.read_bytes()
        elif isinstance(file, BytesIO):
            file = file.getvalue()
        return MediaMessageSegment("file", {"file": file})

    @staticmethod
    def voice(file: Union[Path, BytesIO, bytes], duration: int = 1) -> "MessageSegment":
        if isinstance(file, Path):
            file = file.read_bytes()
        elif isinstance(file, BytesIO):
            file = file.getvalue()
        return MediaMessageSegment("voice", {"file": file, "duration": duration})

    @staticmethod
    def video(file: Union[Path, BytesIO, bytes]) -> "MessageSegment":
        if isinstance(file, Path):
            file = file.read_bytes()
        elif isinstance(file, BytesIO):
            file = file.getvalue()
        return MediaMessageSegment("video", {"file": file})

    @staticmethod
    def face(face_id: str) -> "MessageSegment":
        return MessageSegment("face", {"face_id": face_id})

    @staticmethod
    def reply(
        message_seq: str,
        message_id: Optional[str] = None,
        sender_uin: Optional[str] = None,
    ) -> "MessageSegment":
        return MessageSegment(
            "reply",
            {"msg_id": message_id, "msg_seq": message_seq, "sender_uin": sender_uin},
        )

    @staticmethod
    def ark(data: str) -> "MessageSegment":
        return MessageSegment("ark", {"data": data})

    @staticmethod
    def market_face(
        package_id: str, emoji_id: str, face_name: str, key: str, face_path: str
    ) -> "MessageSegment":
        log("WARNING", "market_face only can be received!")
        return MessageSegment(
            "market_face",
            {
                "package_id": package_id,
                "emoji_id": emoji_id,
                "face_name": face_name,
                "key": key,
                "face_path": face_path,
            },
        )

    @staticmethod
    def forward(xml: str, id: str, file_name: str) -> "MessageSegment":
        log(
            "WARNING",
            "forward only can be received!\n"
            "using `bot.send_fake_forward` to send a fake forward msg",
        )
        return MessageSegment(
            "forward",
            {"xml": xml, "id": id, "name": file_name},
        )


class MediaMessageSegment(MessageSegment):
    async def download(self, bot: "Bot") -> bytes:
        path = Path(self.data["path"])
        if path.exists():
            with path.open("rb") as f:
                return f.read()
        resp = await bot.adapter.request(
            Request(
                "POST",
                bot.info.api_base / "message" / "fetchRichMedia",
                headers={"Authorization": f"Bearer {bot.info.token}"},
                json={
                    "msgId": self.data["_msg_id"],
                    "chatType": self.data["_chat_type"],
                    "peerUid": self.data["_peer_uin"],
                    "elementId": self.data["id"],
                    "thumbSize": 0,
                    "downloadType": 2,
                },
            )
        )
        if resp.status_code == 200:
            return resp.content  # type: ignore
        raise NetworkError("red", resp)

    async def upload(self, bot: "Bot") -> UploadResponse:
        data = self.data["file"] if self.data.get("file") else await self.download(bot)
        resp = await bot.adapter.request(
            Request(
                "POST",
                bot.info.api_base / "upload",
                headers={
                    "Authorization": f"Bearer {bot.info.token}",
                },
                files={f"file_{self.type}": (f"file_{self.type}", data)},
            )
        )
        return UploadResponse.parse_raw(resp.content)


class Message(BaseMessage[MessageSegment]):
    @classmethod
    @override
    def get_segment_class(cls) -> Type[MessageSegment]:
        # 返回适配器的 MessageSegment 类型本身
        return MessageSegment

    @staticmethod
    @override
    def _construct(msg: str) -> Iterable[MessageSegment]:
        yield MessageSegment.text(msg)

    @classmethod
    def from_red_message(
        cls, message: List[Element], msg_id: str, chat_type: int, peer_uin: str
    ) -> "Message":
        msg = Message()
        for element in message:
            if element.elementType == 1:
                if TYPE_CHECKING:
                    assert element.textElement
                text = element.textElement
                if not text.atType:
                    msg.append(MessageSegment.text(text.content))
                elif text.atType == 1:
                    msg.append(MessageSegment.at_all())
                elif text.atType == 2:
                    msg.append(
                        MessageSegment.at(
                            text.atNtUin or text.atNtUid, text.content[1:]
                        )
                    )  # type: ignore  # noqa: E501
            if element.elementType == 2:
                if TYPE_CHECKING:
                    assert element.picElement
                pic = element.picElement
                msg.append(
                    MediaMessageSegment(
                        "image",
                        {
                            "md5": pic.md5HexStr,
                            "size": pic.fileSize,
                            "id": element.elementId,
                            "uuid": pic.fileUuid,
                            "path": pic.sourcePath,
                            "width": pic.picWidth,
                            "height": pic.picHeight,
                            "_msg_id": msg_id,
                            "_chat_type": chat_type,
                            "_peer_uin": peer_uin,
                        },
                    )
                )
            if element.elementType == 3:
                if TYPE_CHECKING:
                    assert element.fileElement
                file = element.fileElement
                msg.append(
                    MediaMessageSegment(
                        "file",
                        {
                            "id": element.elementId,
                            "md5": file.fileMd5,
                            "name": file.fileName,
                            "size": file.fileSize,
                            "uuid": file.fileUuid,
                            "_msg_id": msg_id,
                            "_chat_type": chat_type,
                            "_peer_uin": peer_uin,
                        },
                    )
                )
            if element.elementType == 4:
                if TYPE_CHECKING:
                    assert element.pttElement
                ptt = element.pttElement
                msg.append(
                    MediaMessageSegment(
                        "voice",
                        {
                            "id": element.elementId,
                            "name": ptt.fileName,
                            "path": ptt.filePath,
                            "md5": ptt.md5HexStr,
                            "type": ptt.voiceChangeType,
                            "text": ptt.text,
                            "duration": ptt.duration,
                            "amplitudes": ptt.waveAmplitudes,
                            "uuid": ptt.fileUuid,
                            "_msg_id": msg_id,
                            "_chat_type": chat_type,
                            "_peer_uin": peer_uin,
                        },
                    )
                )
            if element.elementType == 5:
                if TYPE_CHECKING:
                    assert element.videoElement
                video = element.videoElement
                msg.append(
                    MediaMessageSegment(
                        "video",
                        {
                            "id": element.elementId,
                            "path": video.filePath,
                            "name": video.fileName,
                            "md5": video.videoMd5,
                            "format": video.fileFormat,
                            "time": video.fileTime,
                            "size": video.fileSize,
                            "uuid": video.fileUuid,
                            "thumb_md5": video.thumbMd5,
                            "thumb_size": video.thumbSize,
                            "thumb_width": video.thumbWidth,
                            "thumb_height": video.thumbHeight,
                            "thumb_path": video.thumbPath,
                            "busiType": video.busiType,
                            "subBusiType": video.subBusiType,
                            "transferStatus": video.transferStatus,
                            "progress": video.progress,
                            "invalidState": video.invalidState,
                            "fileSubId": video.fileSubId,
                            "fileBizId": video.fileBizId,
                            "_msg_id": msg_id,
                            "_chat_type": chat_type,
                            "_peer_uin": peer_uin,
                        },
                    )
                )
            if element.elementType == 6:
                if TYPE_CHECKING:
                    assert element.faceElement
                face = element.faceElement
                msg.append(MessageSegment.face(str(face.faceIndex)))
            if element.elementType == 7:
                if TYPE_CHECKING:
                    assert element.replyElement
                reply = element.replyElement
                msg.append(
                    MessageSegment(
                        "reply",
                        {
                            "_origin": reply,
                            "msg_id": reply.sourceMsgIdInRecords,
                            "msg_seq": reply.replayMsgSeq,
                            # reply 元素仍然只有 senderUid
                            "sender_uin": reply.senderUid,
                        },
                    )
                )
            if element.elementType == 10:
                if TYPE_CHECKING:
                    assert element.arkElement
                ark = element.arkElement
                msg.append(MessageSegment.ark(ark.bytesData))
            if element.elementType == 11:
                if TYPE_CHECKING:
                    assert element.marketFaceElement
                market_face = element.marketFaceElement
                msg.append(
                    MessageSegment(
                        "market_face",
                        {
                            "package_id": market_face.emojiPackageId,
                            "face_name": market_face.faceName,
                            "emoji_id": market_face.emojiId,
                            "key": market_face.key,
                            "static_path": market_face.staticFacePath,
                            "dynamic_path": market_face.dynamicFacePath,
                        },
                    )
                )
            if element.elementType == 16:
                if TYPE_CHECKING:
                    assert element.multiForwardMsgElement
                forward_msg = element.multiForwardMsgElement
                msg.append(
                    MessageSegment(
                        "forward",
                        {
                            "xml": forward_msg.xmlContent,
                            "id": forward_msg.resId,
                            "name": forward_msg.fileName,
                        },
                    )
                )
        return msg

    async def export(self, bot: "Bot") -> List[dict]:
        res = []
        for seg in self:
            if seg.type == "text":
                res.append(
                    {"elementType": 1, "textElement": {"content": seg.data["text"]}}
                )
            elif seg.type == "at":
                res.append(
                    {
                        "elementType": 1,
                        "textElement": {
                            "atType": 2,
                            "atNtUin": seg.data["user_id"],
                            "content": f"@{seg.data['user_name'] or seg.data['user_id']}",  # noqa: E501
                        },
                    }
                )
            elif seg.type == "at_all":
                res.append({"elementType": 1, "textElement": {"atType": 1}})
            elif seg.type == "image":
                if TYPE_CHECKING:
                    assert isinstance(seg, MediaMessageSegment)
                resp = await seg.upload(bot)
                file = Path(resp.ntFilePath)
                res.append(
                    {
                        "elementType": 2,
                        "picElement": {
                            "original": True,
                            "md5HexStr": resp.md5,
                            "picWidth": resp.imageInfo and resp.imageInfo.width,
                            "picHeight": resp.imageInfo and resp.imageInfo.height,
                            "fileSize": resp.fileSize,
                            "fileName": file.name,
                            "sourcePath": resp.ntFilePath,
                        },
                    }
                )
            elif seg.type == "file":
                if TYPE_CHECKING:
                    assert isinstance(seg, MediaMessageSegment)
                resp = await seg.upload(bot)
                file = Path(resp.ntFilePath)
                res.append(
                    {
                        "elementType": 3,
                        "fileElement": {
                            "fileMd5": resp.md5,
                            "fileSize": resp.fileSize,
                            "fileName": file.name,
                            "filePath": resp.ntFilePath,
                            "picHeight": 0,
                            "picWidth": 0,
                            "picThumbPath": {},
                            "file10MMd5": "",
                            "fileSha": "",
                            "fileSha3": "",
                            "fileUuid": "",
                            "fileSubId": "",
                            "thumbFileSize": 750,
                        },
                    }
                )
            elif seg.type == "voice":
                if TYPE_CHECKING:
                    assert isinstance(seg, MediaMessageSegment)
                resp = await seg.upload(bot)
                file = Path(resp.ntFilePath)
                res.append(
                    {
                        "elementType": 4,
                        "pttElement": {
                            "canConvert2Text": True,
                            "md5HexStr": resp.md5,
                            "fileSize": resp.fileSize,
                            "fileName": file.name,
                            "filePath": resp.ntFilePath,
                            "duration": seg.data["duration"],
                            "formatType": 1,
                            "voiceType": 1,
                            "voiceChangeType": 0,
                            "playState": 1,
                            "waveAmplitudes": seg.get(
                                "amplitudes", [99 for _ in range(17)]
                            ),
                        },
                    }
                )
            elif seg.type == "video":
                raise NotImplementedError(
                    "Unsupported MessageSegment type: " f"{seg.type}"
                )
            elif seg.type == "face":
                res.append(
                    {
                        "elementType": 6,
                        "faceElement": {"faceIndex": seg.data["face_id"]},
                    }
                )
            elif seg.type == "reply":
                res.append(
                    {
                        "elementType": 7,
                        "replyElement": {
                            "replayMsgId": seg.data["msg_id"],
                            "replayMsgSeq": seg.data["msg_seq"],
                            "senderUin": seg.data["sender_uin"],
                            "senderUinStr": str(seg.data["sender_uin"]),
                        },
                    }
                )
            elif seg.type == "ark":
                res.append(
                    {"elementType": 10, "arkElement": {"bytesData": seg.data["data"]}}
                )
            elif seg.type == "market_face":
                raise NotImplementedError(
                    "Unsupported MessageSegment type: " f"{seg.type}"
                )
            elif seg.type == "forward_msg":
                raise NotImplementedError(
                    "Unsupported MessageSegment type: " f"{seg.type}"
                )
        return res


@dataclass
class ForwardNode:
    uin: str
    name: str
    group: Union[int, str]
    message: Message
    time: datetime = field(default_factory=datetime.now)

    async def export(self, seq: int, bot: "Bot") -> dict:
        elems = []
        for seg in self.message:
            if seg.type == "text":
                elems.append({"text": {"str": seg.data["text"]}})
            elif seg.type == "at":
                elems.append({"text": {"str": f"@{seg.data['user_id']}"}})
            elif seg.type == "at_all":
                elems.append({"text": {"str": "@全体成员"}})
            elif seg.type == "image":
                if TYPE_CHECKING:
                    assert isinstance(seg, MediaMessageSegment)
                resp = await seg.upload(bot)
                md5 = resp.md5
                file = Path(resp.ntFilePath)
                pid = f"{{{md5[:8].upper()}-{md5[8:12].upper()}-{md5[12:16].upper()}-{md5[16:20].upper()}-{md5[20:].upper()}}}{file.suffix}"  # noqa: E501
                elems.append(
                    {
                        "customFace": {
                            "filePath": pid,
                            "fileId": random.randint(0, 65535),
                            "serverIp": -1740138629,
                            "serverPort": 80,
                            "fileType": 1001,
                            "useful": 1,
                            "md5": [int(md5[i : i + 2], 16) for i in range(0, 32, 2)],
                            "imageType": 1001,
                            "width": resp.imageInfo and resp.imageInfo.width,
                            "height": resp.imageInfo and resp.imageInfo.height,
                            "size": resp.fileSize,
                            "origin": 0,
                            "thumbWidth": 0,
                            "thumbHeight": 0,
                            "pbReserve": [2, 0],
                        }
                    }
                )
            else:
                elems.append({"text": {"str": f"[{seg.type}]"}})
        return {
            "head": {
                "field2": self.uin,
                "field8": {
                    "field1": int(self.group),
                    "field4": self.name,
                },
            },
            "content": {
                "field1": 82,
                "field4": random.randint(0, 4294967295),
                "field5": seq,
                "field6": int(self.time.timestamp()),
                "field7": 1,
                "field8": 0,
                "field9": 0,
                "field15": {"field1": 0, "field2": 0},
            },
            "body": {"richText": {"elems": elems}},
        }



import os
from pathlib import Path
from typing import Dict, List

from yarl import URL
from pydantic import Extra, Field, BaseModel


class BotInfo(BaseModel):
    host: str = "localhost"
    port: int
    token: str

    @property
    def api_base(self):
        return URL(f"http://{self.host}:{self.port}") / "api"


class Server(BaseModel):
    type: str
    token: str
    port: int = 16530
    enable: bool = True
    host: str = Field(default="localhost", alias="listen")


class Servers(BaseModel, extra=Extra.ignore):
    servers: List[Server] = Field(default_factory=list)
    enable: bool = True


class ChronocatConfig(Servers, extra=Extra.ignore):
    overrides: Dict[str, Servers] = Field(default_factory=dict)


class Config(BaseModel, extra=Extra.ignore):
    red_bots: List[BotInfo] = Field(default_factory=list)
    """bot 配置"""

    red_auto_detect: bool = False
    """是否自动检测 chronocat 配置，默认为 False"""


# get `home` path
home = Path(os.path.expanduser("~"))
# get `config` path
config = home / ".chronocat" / "config" / "chronocat.yml"


def get_config() -> List[BotInfo]:
    import yaml

    if not config.exists():
        return []
    with open(config, encoding="utf-8") as f:
        chrono_config = ChronocatConfig.parse_obj(yaml.safe_load(f))
    base_config = next(
        (s for s in chrono_config.servers if s.type == "red" and s.enable), None
    )
    if (
        not chrono_config.overrides
        or len(chrono_config.overrides) == 1
        and "10000" in chrono_config.overrides
    ):
        return [
            BotInfo(
                port=base_config.port, token=base_config.token, host=base_config.host
            )
        ]
    return [
        BotInfo(port=server.port, token=server.token, host=server.host)
        for servers in chrono_config.overrides.values()
        for server in servers.servers
        if servers.enable and server.type == "red" and server.enable
    ]



from .bot import Bot as Bot
from .adapter import Adapter as Adapter
from .message import Message as Message
from .event import MessageEvent as MessageEvent
from .message import MessageSegment as MessageSegment
from .event import GroupMessageEvent as GroupMessageEvent
from .event import PrivateMessageEvent as PrivateMessageEvent

__version__ = "0.5.0"



from enum import IntEnum
from typing import Any, List, Optional

from pydantic import BaseModel


class ChatType(IntEnum):
    FRIEND = 1
    GROUP = 2


class RoleInfo(BaseModel):
    roleId: str
    name: str
    color: int


class EmojiAd(BaseModel):
    url: str
    desc: str


class EmojiMall(BaseModel):
    packageId: int
    emojiId: int


class EmojiZplan(BaseModel):
    actionId: int
    actionName: str
    actionType: int
    playerNumber: int
    peerUid: str
    bytesReserveInfo: str


class OtherAdd(BaseModel):
    uid: Optional[str]
    name: Optional[str]
    uin: Optional[str]


class MemberAdd(BaseModel):
    showType: int
    otherAdd: Optional[OtherAdd]
    otherAddByOtherQRCode: Optional[Any]
    otherAddByYourQRCode: Optional[Any]
    youAddByOtherQRCode: Optional[Any]
    otherInviteOther: Optional[Any]
    otherInviteYou: Optional[Any]
    youInviteOther: Optional[Any]


class ShutUpTarget(BaseModel):
    uid: Optional[str]
    card: str
    name: str
    role: int
    uin: str


class ShutUp(BaseModel):
    curTime: int
    duration: int
    admin: ShutUpTarget
    member: ShutUpTarget


class GroupElement(BaseModel):
    type: int
    role: int
    groupName: Optional[str]
    memberUid: Optional[str]
    memberNick: Optional[str]
    memberRemark: Optional[str]
    adminUid: Optional[str]
    adminNick: Optional[str]
    adminRemark: Optional[str]
    createGroup: Optional[Any]
    memberAdd: Optional[MemberAdd]
    shutUp: Optional[ShutUp]
    memberUin: Optional[str]
    adminUin: Optional[str]


class XmlElement(BaseModel):
    busiType: Optional[str]
    busiId: Optional[str]
    c2cType: int
    serviceType: int
    ctrlFlag: int
    content: Optional[str]
    templId: Optional[str]
    seqId: Optional[str]
    templParam: Optional[Any]
    pbReserv: Optional[str]
    members: Optional[Any]


class TextElement(BaseModel):
    content: str
    atType: Optional[int]
    atUid: Optional[str]
    atTinyId: Optional[str]
    atNtUid: Optional[str]
    atNtUin: Optional[str]
    subElementType: Optional[int]
    atChannelId: Optional[str]
    atRoleId: Optional[str]
    atRoleColor: Optional[str]
    atRoleName: Optional[str]
    needNotify: Optional[str]


class PicElement(BaseModel):
    picSubType: Optional[int]
    fileName: str
    fileSize: str
    picWidth: Optional[int]
    picHeight: Optional[int]
    original: Optional[bool]
    md5HexStr: str
    sourcePath: str
    thumbPath: Optional[Any]
    transferStatus: Optional[int]
    progress: Optional[int]
    picType: Optional[int]
    invalidState: Optional[int]
    fileUuid: Optional[str]
    fileSubId: Optional[str]
    thumbFileSize: Optional[int]
    summary: Optional[str]
    emojiAd: Optional[EmojiAd]
    emojiMall: Optional[EmojiMall]
    emojiZplan: Optional[EmojiZplan]


class FaceElement(BaseModel):
    faceIndex: int
    faceText: Optional[str]
    """{None: normal, '/xxx': sticker, '': poke}"""
    faceType: int
    """{1: normal, 2: normal-extended, 3: sticker, 5: poke}"""
    packId: Optional[Any]
    stickerId: Optional[Any]
    sourceType: Optional[Any]
    stickerType: Optional[Any]
    resultId: Optional[Any]
    surpriseId: Optional[Any]
    randomType: Optional[Any]
    imageType: Optional[Any]
    pokeType: Optional[Any]
    spokeSummary: Optional[Any]
    doubleHit: Optional[Any]
    vaspokeId: Optional[Any]
    vaspokeName: Optional[Any]
    vaspokeMinver: Optional[Any]
    pokeStrength: Optional[Any]
    msgType: Optional[Any]
    faceBubbleCount: Optional[Any]
    pokeFlag: Optional[Any]


class FileElement(BaseModel):
    fileMd5: str
    fileName: str
    filePath: str
    fileSize: str
    picHeight: Optional[int]
    picWidth: Optional[int]
    picThumbPath: Optional[Any]
    expireTime: Optional[str]
    file10MMd5: Optional[str]
    fileSha: Optional[str]
    fileSha3: Optional[str]
    videoDuration: Optional[int]
    transferStatus: Optional[int]
    progress: Optional[int]
    invalidState: Optional[int]
    fileUuid: Optional[str]
    fileSubId: Optional[str]
    thumbFileSize: Optional[int]
    fileBizId: Optional[Any]
    thumbMd5: Optional[Any]
    folderId: Optional[Any]
    fileGroupIndex: Optional[int]
    fileTransType: Optional[Any]


class PttElement(BaseModel):
    fileName: str
    filePath: str
    md5HexStr: str
    fileSize: str
    duration: int
    formatType: int
    voiceType: int
    voiceChangeType: int
    canConvert2Text: bool
    fileId: int
    fileUuid: str
    text: Optional[str]
    translateStatus: Optional[int]
    transferStatus: Optional[int]
    progress: Optional[int]
    playState: Optional[int]
    waveAmplitudes: Optional[List[int]]
    invalidState: Optional[int]
    fileSubId: Optional[str]
    fileBizId: Optional[Any]


class VideoElement(BaseModel):
    filePath: str
    fileName: str
    videoMd5: str
    thumbMd5: str
    fileTime: int
    thumbSize: int
    fileFormat: int
    fileSize: str
    thumbWidth: int
    thumbHeight: int
    busiType: int
    subBusiType: int
    thumbPath: Optional[Any]
    transferStatus: Optional[int]
    progress: Optional[int]
    invalidState: Optional[int]
    fileUuid: Optional[str]
    fileSubId: Optional[str]
    fileBizId: Optional[Any]


class ReplyElement(BaseModel):
    replayMsgId: Optional[str]
    replayMsgSeq: str
    replyMsgTime: Optional[str]
    sourceMsgIdInRecords: str
    sourceMsgTextElems: Optional[Any]
    senderUid: Optional[str]
    senderUidStr: Optional[str]
    senderUin: Optional[str]


class ArkElement(BaseModel):
    bytesData: str
    """application/json"""


class MarketFaceElement(BaseModel):
    itemType: int
    faceInfo: int
    emojiPackageId: str
    subType: int
    faceName: str
    emojiId: str
    key: str
    staticFacePath: str
    dynamicFacePath: str


class MultiForwardMsgElement(BaseModel):
    xmlContent: str
    resId: str
    fileName: str


class GrayTipElement(BaseModel):
    subElementType: Optional[int]
    revokeElement: Optional[dict]
    proclamationElement: Optional[dict]
    emojiReplyElement: Optional[dict]
    groupElement: Optional[GroupElement]
    buddyElement: Optional[dict]
    feedMsgElement: Optional[dict]
    essenceElement: Optional[dict]
    groupNotifyElement: Optional[dict]
    buddyNotifyElement: Optional[dict]
    xmlElement: Optional[XmlElement]
    fileReceiptElement: Optional[dict]
    localGrayTipElement: Optional[dict]
    blockGrayTipElement: Optional[dict]
    aioOpGrayTipElement: Optional[dict]
    jsonGrayTipElement: Optional[dict]


class Element(BaseModel):
    elementType: int
    elementId: Optional[str]
    extBufForUI: Optional[str]
    picElement: Optional[PicElement]
    textElement: Optional[TextElement]
    arkElement: Optional[ArkElement]
    avRecordElement: Optional[dict]
    calendarElement: Optional[dict]
    faceElement: Optional[FaceElement]
    fileElement: Optional[FileElement]
    giphyElement: Optional[dict]
    grayTipElement: Optional[GrayTipElement]
    inlineKeyboardElement: Optional[dict]
    liveGiftElement: Optional[dict]
    markdownElement: Optional[dict]
    marketFaceElement: Optional[MarketFaceElement]
    multiForwardMsgElement: Optional[MultiForwardMsgElement]
    pttElement: Optional[PttElement]
    replyElement: Optional[ReplyElement]
    structLongMsgElement: Optional[dict]
    textGiftElement: Optional[dict]
    videoElement: Optional[VideoElement]
    walletElement: Optional[dict]
    yoloGameResultElement: Optional[dict]


class MsgType(IntEnum):
    normal = 2
    may_file = 3
    system = 5
    voice = 6
    video = 7
    value8 = 8
    reply = 9
    wallet = 10
    ark = 11
    may_market = 17


class Message(BaseModel):
    msgId: str
    msgRandom: str
    msgSeq: str
    cntSeq: str
    chatType: ChatType
    msgType: MsgType
    subMsgType: int
    sendType: int
    senderUid: Optional[str]
    senderUin: Optional[str]
    peerUid: Optional[str]
    peerUin: Optional[str]
    channelId: str
    guildId: str
    guildCode: str
    fromUid: str
    fromAppid: str
    msgTime: str
    msgMeta: str
    sendStatus: int
    sendMemberName: str
    sendNickName: str
    guildName: str
    channelName: str
    elements: List[Element]
    records: List["Message"]
    emojiLikesList: List[Any]
    commentCnt: str
    directMsgFlag: int
    directMsgMembers: List[Any]
    peerName: str
    editable: bool
    avatarMeta: str
    avatarPendant: Optional[str]
    feedId: Optional[str]
    roleId: str
    timeStamp: str
    isImportMsg: bool
    atType: int
    roleType: Optional[int]
    fromChannelRoleInfo: RoleInfo
    fromGuildRoleInfo: RoleInfo
    levelRoleInfo: RoleInfo
    recallTime: str
    isOnlineMsg: bool
    generalFlags: str
    clientSeq: str
    nameType: Optional[int]
    avatarFlag: Optional[int]


class Profile(BaseModel):
    uid: str
    qid: str
    uin: str
    nick: str
    remark: str
    longNick: str
    avatarUrl: str
    birthday_year: int
    birthday_month: int
    birthday_day: int
    sex: int
    topTime: str
    isBlock: bool
    isMsgDisturb: bool
    isSpecialCareOpen: bool
    isSpecialCareZone: bool
    ringId: str
    status: int
    extStatus: int
    categoryId: int
    onlyChat: bool
    qzoneNotWatch: bool
    qzoneNotWatched: bool
    vipFlag: bool
    yearVipFlag: bool
    svipFlag: bool
    vipLevel: int


class Member(BaseModel):
    uid: str
    qid: str
    uin: str
    nick: str
    remark: str
    cardType: int
    cardName: str
    role: int
    avatarPath: str
    shutUpTime: int
    isDelete: bool


class Group(BaseModel):
    groupCode: str
    maxMember: int
    memberCount: int
    groupName: str
    groupStatus: int
    memberRole: int
    isTop: bool
    toppedTimestamp: str
    privilegeFlag: int
    isConf: bool
    hasModifyConfGroupFace: bool
    hasModifyConfGroupName: bool
    remarkName: str
    avatarUrl: str
    hasMemo: bool
    groupShutupExpireTime: str
    personShutupExpireTime: str
    discussToGroupUin: str
    discussToGroupMaxMsgSeq: int
    discussToGroupTime: int


class ImageInfo(BaseModel):
    width: int
    height: int
    type: Optional[str]
    mime: Optional[str]
    wUnits: Optional[str]
    hUnits: Optional[str]


class UploadResponse(BaseModel):
    md5: str
    imageInfo: Optional[ImageInfo]
    fileSize: int
    filePath: str
    ntFilePath: str



import re
from copy import deepcopy
from typing import Any, Dict, Optional
from typing_extensions import override
from datetime import datetime, timedelta

from nonebot.utils import escape_tag
from pydantic.class_validators import root_validator

from nonebot.adapters import Event as BaseEvent

from .message import Message
from .api.model import Message as MessageModel
from .api.model import MsgType, ChatType, ReplyElement, ShutUpTarget


class Event(BaseEvent):
    @override
    def get_type(self) -> str:
        # 现阶段Red协议只有message事件
        return "event"

    @override
    def get_event_name(self) -> str:
        # 返回事件的名称，用于日志打印
        return "event"

    @override
    def get_event_description(self) -> str:
        return escape_tag(str(self.dict()))

    @override
    def get_message(self):
        raise ValueError("Event has no message!")

    @override
    def get_user_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def get_session_id(self) -> str:
        raise ValueError("Event has no context!")

    @override
    def is_tome(self) -> bool:
        raise ValueError("Event has no context!")

    @classmethod
    def convert(cls, obj: Any):
        """将 Red API 返回的数据转换为对应的 Model 类

        子类可根据需要重写此方法
        """
        return cls.parse_obj(obj)


class MessageEvent(Event, MessageModel):
    """消息事件"""

    to_me: bool = False
    """
    :说明: 消息是否与机器人有关

    :类型: ``bool``
    """
    reply: Optional[ReplyElement] = None
    """
    :说明: 消息中提取的回复消息，内容为 ``get_msg`` API 返回结果

    :类型: ``Optional[ReplyElement]``
    """
    message: Message
    original_message: Message

    @override
    def get_type(self) -> str:
        return "message"

    @override
    def get_event_name(self) -> str:
        # 返回事件的名称，用于日志打印
        return "message"

    @override
    def get_message(self) -> Message:
        return self.message

    @root_validator(pre=True, allow_reuse=True)
    def check_message(cls, values: Dict[str, Any]) -> Dict[str, Any]:
        if "elements" in values:
            values["message"] = Message.from_red_message(
                values["elements"],
                values["msgId"],
                values["chatType"],
                values["peerUin"] or values["peerUid"],
            )
            values["original_message"] = deepcopy(values["message"])
        return values

    @override
    def get_user_id(self) -> str:
        # 获取用户 ID 的方法，根据事件具体实现，如果事件没有用户 ID，则抛出异常
        if self.senderUin is None:
            raise ValueError("user_id doesn't exist.")
        return self.senderUin

    @override
    def get_session_id(self) -> str:
        # 获取事件会话 ID 的方法，根据事件具体实现，如果事件没有相关 ID，则抛出异常
        return f"{self.peerUin or self.peerUid}_{self.senderUin or self.senderUid}"

    @override
    def is_tome(self) -> bool:
        return self.to_me

    @property
    def scene(self) -> str:
        """群组或好友的id"""
        return self.peerUin or self.peerUid

    @property
    def is_group(self) -> bool:
        """是否为群组消息"""
        return self.chatType == ChatType.GROUP

    @property
    def is_private(self) -> bool:
        """是否为私聊消息"""
        return self.chatType == ChatType.FRIEND


class PrivateMessageEvent(MessageEvent):
    """好友消息事件"""

    @override
    def get_event_name(self) -> str:
        return "message.private"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Message from {self.sendNickName or self.senderUin or self.senderUid}: "
            f"{self.get_message()}"
        )
        return escape_tag(text)


class GroupMessageEvent(MessageEvent):
    @override
    def get_event_name(self) -> str:
        return "message.group"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Message from {self.sendMemberName or self.senderUin or self.senderUid} "
            f"in {self.peerName or self.peerUin or self.peerUid}: "
            f"{self.get_message()}"
        )
        return escape_tag(text)


class NoticeEvent(Event):
    msgId: str
    msgRandom: str
    msgSeq: str
    cntSeq: str
    chatType: ChatType
    msgType: MsgType
    subMsgType: int
    peerUid: str
    peerUin: Optional[str]

    @override
    def get_type(self) -> str:
        return "notice"

    @override
    def get_event_name(self) -> str:
        return "notice"

    @property
    def scene(self) -> str:
        """群组或好友的id"""
        return self.peerUin or self.peerUid

    class Config:
        extra = "ignore"


class GroupNameUpdateEvent(NoticeEvent):
    """群名变更事件"""

    currentName: str
    operatorUid: str
    operatorName: str

    @override
    def get_event_name(self) -> str:
        return "notice.group_name_update"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Group {self.peerUin or self.peerUid} name updated to {self.currentName}"
        )
        return escape_tag(text)

    @override
    def get_user_id(self) -> str:
        # 获取用户 ID 的方法，根据事件具体实现，如果事件没有用户 ID，则抛出异常
        if self.operatorUid is None:
            raise ValueError("user_id doesn't exist.")
        return self.operatorUid

    @override
    def get_session_id(self) -> str:
        # 获取事件会话 ID 的方法，根据事件具体实现，如果事件没有相关 ID，则抛出异常
        return f"{self.peerUin or self.peerUid}_{self.operatorUid}"

    @classmethod
    @override
    def convert(cls, obj: Any):
        assert isinstance(obj, MessageModel)
        return cls(
            msgId=obj.msgId,
            msgRandom=obj.msgRandom,
            msgSeq=obj.msgSeq,
            cntSeq=obj.cntSeq,
            chatType=obj.chatType,
            msgType=obj.msgType,
            subMsgType=obj.subMsgType,
            peerUid=obj.peerUid,
            peerUin=obj.peerUin,
            currentName=obj.elements[0].grayTipElement.groupElement.groupName,  # type: ignore  # noqa: E501
            operatorUid=obj.elements[0].grayTipElement.groupElement.memberUin,  # type: ignore  # noqa: E501
            operatorName=obj.elements[0].grayTipElement.groupElement.memberNick,  # type: ignore  # noqa: E501
        )


class MemberAddEvent(NoticeEvent):
    """群成员增加事件"""

    memberUid: str
    operatorUid: str
    memberName: Optional[str]

    @override
    def get_event_name(self) -> str:
        return "notice.member_add"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Member {f'{self.memberName}({self.memberUid})' if self.memberName else self.memberUid} added to "  # noqa: E501
            f"{self.peerUin or self.peerUid}"
        )
        return escape_tag(text)

    @override
    def get_user_id(self) -> str:
        return self.memberUid

    @override
    def get_session_id(self) -> str:
        # 获取事件会话 ID 的方法，根据事件具体实现，如果事件没有相关 ID，则抛出异常
        return f"{self.peerUin or self.peerUid}_{self.memberUid}"

    legacy_invite_message = re.compile(
        r'jp="(\d+)".*jp="(\d+)"', re.DOTALL | re.MULTILINE | re.IGNORECASE
    )

    @classmethod
    @override
    def convert(cls, obj: Any):
        assert isinstance(obj, MessageModel)
        params = {
            "msgId": obj.msgId,
            "msgRandom": obj.msgRandom,
            "msgSeq": obj.msgSeq,
            "cntSeq": obj.cntSeq,
            "chatType": obj.chatType,
            "msgType": obj.msgType,
            "subMsgType": obj.subMsgType,
            "peerUid": obj.peerUid,
            "peerUin": obj.peerUin,
        }
        if obj.elements[0].grayTipElement.xmlElement:  # type: ignore
            if not (
                mat := cls.legacy_invite_message.match(
                    obj.elements[0].grayTipElement.xmlElement.content
                )
            ):  # type: ignore  # noqa: E501
                raise ValueError("Invalid legacy invite message.")
            params["operatorUid"] = mat[1]
            params["memberUid"] = mat[2]
        else:
            params["memberUid"] = obj.elements[0].grayTipElement.groupElement.memberUin  # type: ignore  # noqa: E501
            params["operatorUid"] = obj.elements[0].grayTipElement.groupElement.adminUin  # type: ignore  # noqa: E501
            params["memberName"] = obj.elements[0].grayTipElement.groupElement.memberNick  # type: ignore  # noqa: E501
        return cls(**params)


class MemberMuteEvent(NoticeEvent):
    """群成员禁言相关事件"""

    start: datetime
    duration: timedelta
    operator: ShutUpTarget
    member: ShutUpTarget

    @override
    def get_user_id(self) -> str:
        return self.member.uin or self.member.uid

    @override
    def get_session_id(self) -> str:
        # 获取事件会话 ID 的方法，根据事件具体实现，如果事件没有相关 ID，则抛出异常
        return f"{self.peerUin or self.peerUid}_{self.member.uin or self.member.uid}"

    @classmethod
    @override
    def convert(cls, obj: Any):
        assert isinstance(obj, MessageModel)
        params = {
            "msgId": obj.msgId,
            "msgRandom": obj.msgRandom,
            "msgSeq": obj.msgSeq,
            "cntSeq": obj.cntSeq,
            "chatType": obj.chatType,
            "msgType": obj.msgType,
            "subMsgType": obj.subMsgType,
            "peerUid": obj.peerUid,
            "peerUin": obj.peerUin,
            "start": datetime.fromtimestamp(
                obj.elements[0].grayTipElement.groupElement.shutUp.curTime
            ),
            # type: ignore  # noqa: E501
            "duration": timedelta(
                seconds=obj.elements[0].grayTipElement.groupElement.shutUp.duration
            ),
            # type: ignore  # noqa: E501
            "operator": obj.elements[0].grayTipElement.groupElement.shutUp.admin,  # type: ignore  # noqa: E501
            "member": obj.elements[0].grayTipElement.groupElement.shutUp.member,  # type: ignore  # noqa: E501
        }
        if params["duration"].total_seconds() < 1:
            return MemberUnmuteEvent(**params)
        return MemberMutedEvent(**params)


class MemberMutedEvent(MemberMuteEvent):
    """群成员被禁言事件"""

    @override
    def get_event_name(self) -> str:
        return "notice.member_muted"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Member {self.member.uin or self.member.uid} muted in "
            f"{self.peerUin or self.peerUid} for {self.duration}"
        )
        return escape_tag(text)


class MemberUnmuteEvent(MemberMuteEvent):
    """群成员被解除禁言事件"""

    @override
    def get_event_name(self) -> str:
        return "notice.member_unmute"

    @override
    def get_event_description(self) -> str:
        text = (
            f"Member {self.member.uin or self.member.uid} unmute in "
            f"{self.peerUin or self.peerUid}"
        )
        return escape_tag(text)
